#!/bin/bash

accumsize=16
declare -a  candidates=(  6  2  3  0 \
                         14  2  7  0 )
#declare -a  candidates=( 14  2  7  5 \
#                         14  2  7  6 \
#                         13  5 10 10 \
#                          3  2  6  5 \
#                          6  2  3  9 \
#                          6  2 11  6 \
#                          6  2  5  5 \
#                          6  2  5  9 \
#                          5  2  6  9 )



if [ $accumsize -lt 8 ]; then
	echo Aborted!  Specify size range 8-64.
	echo
	exit
fi

testdir="tests/freq_dist"

if [ ! -d "${testdir}" ]; then
	mkdir "${testdir}"
fi



function  candidate_cases {
	printf "\nBEGIN  ... at "
	date +"%F %T"
	printf "\nFrequency distribution reports of pairs and zero-runs for Xoroshiro$(($accumsize*2))(${accumsize}) engine.\n\n"

	top=${#candidates[*]}   # number of entries in array
	enoughram=$((5 * 2**20))   # amount of spare ram required before running next test case
	count=0
	while [ $count -lt $top ]
	do
		while true
		do
			sleep 2
			spareram=$(awk '/MemAvailable/{print $2}' /proc/meminfo)
			if [ $spareram -gt $enoughram ]; then  # Need more than 4 GB spare for next case
				break
			fi
		done

		a=${candidates[$count]}; count=$(($count + 1))
		b=${candidates[$count]}; count=$(($count + 1))
		c=${candidates[$count]}; count=$(($count + 1))
		d=${candidates[$count]}; count=$(($count + 1))

		if [ $d -eq 0 ]; then  # Single summing
			printf "Distribution test case Xoroshiro$(($accumsize*2))+ [$a $b $c]\n"
		else
			printf "Distribution test case Xoroshiro$(($accumsize*2))++ [$a $b $c $d]\n"
		fi

		# Compile candidate test case
		gcc -o "${testdir}/testcase-dists${accumsize}-a${a}b${b}c${c}d${d}" "src/dist-s16.c" -O3 -DCONSTANT_A=${a} -DCONSTANT_B=${b} -DCONSTANT_C=${c} -DCONSTANT_D=${d}

		if [ $? -ne 0 ]; then
			echo Distribution test case aborted!  Compile error.
			echo
			exit 1
		fi

		# Run test cases, concurrently
		pwdir=$( pwd )
		cd "${testdir}"
		nice -10  "./testcase-dists${accumsize}-a${a}b${b}c${c}d${d}" &
		cd "${pwdir}"
	done

	# Delete the binaries, even though tests are still running
	sleep 2
	count=0
	while [ $count -lt $top ]
	do
		a=${candidates[$count]}; count=$(($count + 1))
		b=${candidates[$count]}; count=$(($count + 1))
		c=${candidates[$count]}; count=$(($count + 1))
		d=${candidates[$count]}; count=$(($count + 1))
		rm "${testdir}/testcase-dists${accumsize}-a${a}b${b}c${c}d${d}"
	done
}


candidate_cases |tee -a "${testdir}/dist-xo-s${accumsize}.log"

